package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.utils.LogWriter;
import ac.ed.lurg.utils.StringTabularReader;

public class CerealFractionsManager {
	protected CompositeCountryManager compositeCountryManager;
	private StringTabularReader cerealFractionReader;

	public CerealFractionsManager(CompositeCountryManager compositeCountryManager) {
		this.compositeCountryManager = compositeCountryManager;
		cerealFractionReader = new StringTabularReader(",", new String[]{"Country", "Item", "food"});
		cerealFractionReader.read(ModelConfig.BASE_CEREAL_DEMAND_FILE);
	}

	public Map<CropType, Double> getBaseCerealFraction(CompositeCountry cc) {

		Map<CropType, Double> cerealDemandMap = new HashMap<CropType, Double>();
		for (SingleCountry c : compositeCountryManager.getAllForCompositeCountry(cc)) {
			Map<CropType, Double> singleCMap = getCerealFraction(c);

			for (CropType crop : CropType.getAllItems()) {
				Double newDemandD = singleCMap.get(crop);
				Double prevDemandD = cerealDemandMap.get(crop);
				
				double newDemand = getDoubleOrZero(newDemandD) + getDoubleOrZero(prevDemandD);
				if (newDemand> 0) cerealDemandMap.put(crop, newDemand);
			}
		}
		
		double totalCerealDemand = 0;
		for (Double d : cerealDemandMap.values())
			totalCerealDemand += getDoubleOrZero(d);

		Map<CropType, Double> cerealFractionMap = new HashMap<CropType, Double>();
		for (Entry<CropType, Double> entry : cerealDemandMap.entrySet()) {
			cerealFractionMap.put(entry.getKey(), getDoubleOrZero(entry.getValue())/totalCerealDemand);
		}

		return cerealFractionMap;
	}

	private double getDoubleOrZero(Double d) {
		return (d==null ? 0.0 : d.doubleValue());
	}
	
	private Map<CropType, Double> getCerealFraction(SingleCountry c) {
		Map<CropType, Double> foodDemandMap = new HashMap<CropType, Double>();

		for (CropType crop : CropType.getCereals()) {
			Map<String, String> queryMap = new HashMap<String, String>();
			queryMap.put("Country", c.getCountryName());
			queryMap.put("Item", crop.getFaoName());
			try {
				Map<String, String> row = cerealFractionReader.querySingle(queryMap);
				String cerealS = row.get("food");
				Double cerealAmt = Double.valueOf(cerealS);
				foodDemandMap.put(crop, cerealAmt);
			}
			catch (Exception e) {
				LogWriter.println("Problem finding cereal demand: " + crop.getFaoName() + ", " + c.getCountryName());
			}
		}
		return foodDemandMap;
	}
}
