package ac.ed.lurg.types;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;

public enum CommodityType {

	CEREALS("Cereals", "cereals", false, 0.0115335812,ModelConfig.CEREALS_SUB_PROPORTION,Double.NaN ),
	OILCROPS("Oilcrops", "oilcrops", false, 0.0243500061,ModelConfig.OILCROPS_SUB_PROPORTION,Double.NaN),
	PULSES("Pulses", "pulses", false, 0.0146872440,ModelConfig.PULSES_SUB_PROPORTION,Double.NaN),
	STARCHY_ROOTS("Starchy Roots", "starchyRoots", false, 0.0034561229,ModelConfig.STARCHY_ROOTS_SUB_PROPORTION,Double.NaN),
	MONOGASTRICS("Monogastrics", "monogastrics", true, 0.0019100435, Double.NaN, ModelConfig.MONOGASTRIC_CHANGE_ANNUAL_RATE),
	RUMINANTS("Ruminants", "ruminants", true, 0.0009951162, Double.NaN, ModelConfig.RUMINANT_CHANGE_ANNUAL_RATE);

	private String faoName;
	private String gamsName;
	private boolean isAnimalProduct;
	private double ejPerMt;
	private double meatSubstitutionProportion;
	private double demandChangeRate;
	
	CommodityType (String faoName, String gamsName, boolean isAnimalProduct, double energyPerTon, double meatSubstitutionProportion,double demandChangeRate) {
		this.faoName = faoName;
		this.gamsName = gamsName;
		this.isAnimalProduct = isAnimalProduct;
		this.ejPerMt = energyPerTon; //energy EJ per Mt
		this.meatSubstitutionProportion = meatSubstitutionProportion;
		this.demandChangeRate =demandChangeRate;
	}
	
	private static final Map<String, CommodityType> faoNameCache = new HashMap<String, CommodityType>();
    static {
        for (CommodityType c : values()) {
        	faoNameCache.put(c.getFaoName(), c);
       }
    }

	public static CommodityType getForFaoName(String faoName) {
		CommodityType commodity = faoNameCache.get(faoName);

		if (commodity == null) 
			throw new RuntimeException("Can't find CommodityType for faoName: " + faoName);
		
		return commodity;
	}
	
	public static Collection<CommodityType> getAllItems() {
		return faoNameCache.values();
	}

	public String getFaoName() {
		return faoName;
	}
	
	public String getGamsName() {
		return gamsName;
	}

	public boolean isAnimalProduct() {
		return isAnimalProduct;
	}
	
	public double getEJPerMT(){
		return ejPerMt;
	}
	
	public double getMeatSubstitutionProportion(){
		return meatSubstitutionProportion;
	}
	
	public double demandChangeRate(){
		return demandChangeRate;
	}
	
	public double getSeedWasteRate() {
		// just waste rate for animal products
		return isAnimalProduct ? ModelConfig.SEED_AND_WASTE_FRACTION/2 : ModelConfig.SEED_AND_WASTE_FRACTION;
	}

}
